@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2005 TADA AB - Taby Sweden
+ * Copyright (c) 2004, 2005, 2006 TADA AB - Taby Sweden
  * Distributed under the terms shown in the file COPYRIGHT
  * found in the root folder of this project or at
  * http://eng.tada.se/osprojects/COPYRIGHT.html
@@ -45,9 +45,9 @@ struct String_
 	Oid elementType;
 };
 
-extern Datum _String_coerceObject(Type self, JNIEnv* env, jobject jstr);
+extern Datum _String_coerceObject(Type self, jobject jstr);
 
-extern jvalue _String_coerceDatum(Type self, JNIEnv* env, Datum value);
+extern jvalue _String_coerceDatum(Type self, Datum value);
 
 #ifdef __cplusplus
 }
