@@ -32,16 +32,18 @@ static PortalCleanupProc s_originalCleanupProc = 0;
 
 static void _pljavaPortalCleanup(Portal portal)
 {
-	jobject jportal = (jobject)HashMap_getByOpaque(s_portalMap, portal);
-	if(jportal != 0)
+	/*
+	 * Remove this object from the cache and clear its
+	 * handle.
+	 */
+	jobject jportal = (jobject)HashMap_removeByOpaque(s_portalMap, portal);
+	if(jportal)
 	{
-		/*
-		 * Remove this object from the cache and clear its
-		 * handle.
-		 */
-		HashMap_removeByOpaque(s_portalMap, portal);
+
 		JNI_setLongField(jportal, s_Portal_pointer, 0);
+		JNI_deleteGlobalRef(jportal);
 	}
+
 	portal->cleanup = s_originalCleanupProc;
 	if(s_originalCleanupProc != 0)
 	{
@@ -108,7 +110,7 @@ void Portal_initialize(void)
 		},
 		{
 		"_fetch",
-	  	"(JZI)I",
+	  	"(JJZI)I",
 	  	Java_org_postgresql_pljava_internal_Portal__1fetch
 		},
 		{
@@ -133,7 +135,7 @@ void Portal_initialize(void)
 		},
 		{
 		"_move",
-	  	"(JZI)I",
+	  	"(JJZI)I",
 	  	Java_org_postgresql_pljava_internal_Portal__1move
 		},
 		{ 0, 0, 0 }
@@ -171,16 +173,19 @@ Java_org_postgresql_pljava_internal_Portal__1getPortalPos(JNIEnv* env, jclass cl
 /*
  * Class:     org_postgresql_pljava_internal_Portal
  * Method:    _fetch
- * Signature: (JZI)I
+ * Signature: (JJZI)I
  */
 JNIEXPORT jint JNICALL
-Java_org_postgresql_pljava_internal_Portal__1fetch(JNIEnv* env, jclass clazz, jlong _this, jboolean forward, jint count)
+Java_org_postgresql_pljava_internal_Portal__1fetch(JNIEnv* env, jclass clazz, jlong _this, jlong threadId, jboolean forward, jint count)
 {
 	jint result = 0;
 	if(_this != 0)
 	{
 		BEGIN_NATIVE
 		Ptr2Long p2l;
+		STACK_BASE_VARS
+		STACK_BASE_PUSH(threadId)
+
 		p2l.longVal = _this;
 		PG_TRY();
 		{
@@ -192,6 +197,7 @@ Java_org_postgresql_pljava_internal_Portal__1fetch(JNIEnv* env, jclass clazz, jl
 			Exception_throw_ERROR("SPI_cursor_fetch");
 		}
 		PG_END_TRY();
+		STACK_BASE_POP()
 		END_NATIVE
 	}
 	return result;
@@ -258,7 +264,13 @@ Java_org_postgresql_pljava_internal_Portal__1close(JNIEnv* env, jclass clazz, jl
 		/* Reset our own cleanup callback if needed. No need to come in
 		 * the backway
 		 */
-		HashMap_removeByOpaque(s_portalMap, portal);
+
+		jobject jportal = (jobject)HashMap_removeByOpaque(s_portalMap, portal);
+		if(jportal)
+		{
+			JNI_deleteGlobalRef(jportal);
+		}
+
 		if(portal->cleanup == _pljavaPortalCleanup)
 			portal->cleanup = s_originalCleanupProc;
 
@@ -325,16 +337,19 @@ Java_org_postgresql_pljava_internal_Portal__1isPosOverflow(JNIEnv* env, jclass c
 /*
  * Class:     org_postgresql_pljava_internal_Portal
  * Method:    _move
- * Signature: (ZI)I
+ * Signature: (JJZI)I
  */
 JNIEXPORT jint JNICALL
-Java_org_postgresql_pljava_internal_Portal__1move(JNIEnv* env, jclass clazz, jlong _this, jboolean forward, jint count)
+Java_org_postgresql_pljava_internal_Portal__1move(JNIEnv* env, jclass clazz, jlong _this, jlong threadId, jboolean forward, jint count)
 {
 	jint result = 0;
 	if(_this != 0)
 	{
 		BEGIN_NATIVE
 		Ptr2Long p2l;
+		STACK_BASE_VARS
+		STACK_BASE_PUSH(threadId)
+
 		p2l.longVal = _this;
 		PG_TRY();
 		{
@@ -346,6 +361,7 @@ Java_org_postgresql_pljava_internal_Portal__1move(JNIEnv* env, jclass clazz, jlo
 			Exception_throw_ERROR("SPI_cursor_move");
 		}
 		PG_END_TRY();
+		STACK_BASE_POP()
 		END_NATIVE
 	}
 	return result;
